@@ -1,10 +1,12 @@
-import fileinput
 import logging
 import os
 import re
-import sys
+
+from lxml.etree import XMLSyntaxError
 
 from errors import *
+from lib.thetvdb import TheTvDb as thetvdb
+from lib.tvrage import TvRage as tvrage
 
 log = logging.getLogger('Core')
 
@@ -46,8 +48,8 @@ class TvRenamr():
         Looks at the file given and extracts from it the show title, it's
         season number and episode number using regular expression magic.
         The default formats accepted are: series.0x00.xxx or series.s0e00.xxx
-        or series.000.xxx A user can specify their own regular expression for
-        a format not already covered.
+        A user can specify their own regular expression for a format not
+        already covered.
 
         :param fn: The file name passed in.
         :type fn: String.
@@ -71,73 +73,86 @@ class TvRenamr():
         m = re.compile(regex).match(fn)
         if m is not None:
             show = m.group('show').replace('.', ' ').strip()
-            log.debug('Returned show: %s, season: %s, episode: %s, extension: '
-                        '%s' % (show, m.group('season'), \
-                        m.group('episode'), os.path.splitext(fn)[1]))
-            return {'show': show, 'season': m.group('season'), \
-                    'episode': m.group('episode'), \
-                    'extension': os.path.splitext(fn)[1]}
+            ext = os.path.splitext(fn)[1]
+            details = (show, m.group('season'), m.group('episode'), ext)
+            log.debug('Returned show: %s, season: %s, episode: %s, extension: %s' % details)
+            return details
         else:
             raise UnexpectedFormatException(fn)
 
-    def retrieve_episode_name(self, library='thetvdb', \
-                                canonical=None, **kwargs):
+    def retrieve_episode_name(self, episode, library='thetvdb', canonical=None):
         """
         Retrieves the name of a given episode. The series name, season and
         episode numbers must be specified to get the episode's name. The
-        library can be specified by the user, but will default to The Tv DB.
+        library specified by the user will be used first but will fall back
+        to the other library if an error occurs.
+
+        The first library defaults to The Tv DB.
 
         :param the:
-        :param library: The library to search in.
+        :param library: The library to search.
 
         :returns: The episode title.
         :rtype: A string.
         """
-        if library == 'thetvdb':
-            from lib.thetvdb import TheTvDb as library
-            log.debug('Imported The Tv Db library')
-        elif library == 'tvrage':
-            from lib.tvrage import TvRage as library
-            log.debug('Imported Tv Rage library')
+        libraries = [
+            ('thetvdb', thetvdb),
+            ('tvrage', tvrage),
+        ]
+        choice = None
+        for i, lib in enumerate(libraries):
+            if lib[0] == library:
+                choice = lib
+                libraries.pop(i)
+                break
+        libraries.insert(0, choice)
 
         if canonical:
-            kwargs['show'] = canonical
+            episode.show = canonical
         else:
-            kwargs['show'] = self.config.get_canonical(kwargs['show'])
-        log.debug('Show Name: %s' % kwargs['show'])
+            episode.show = self.config.get_canonical(episode.show)
+        log.debug('Show Name: %s' % episode.show)
 
-        self.library = library(kwargs['show'], kwargs['season'],
-                                kwargs['episode'])
+        for lib in libraries:
+            try:
+                log.debug('Using %s' % lib[1].__name__)
+                self.library = lib[1](episode.show, episode.season, episode.episode)
+                break
+            except XMLSyntaxError:
+                if lib == libraries[-1]:
+                    raise NoMoreLibrariesException
+                log.debug('Got broken XML from %s. Falling back to next library' % lib[1].__name__)
+                continue
 
-        self.title = self.library.get_title()
+        self.title = self.library.title
         log.info('Episode: %s' % self.title)
 
         return self.title
 
-    def format_show_name(self, show, the=None, override=None):
+    def format_show_name(self, show_name, the=None, override=None):
         if the is None:
-            the = self.config.get(show, 'the')
+            the = self.config.get(show_name, 'the')
 
         try:
-            show = self.config.get_output(show)
-            log.debug('Using config output name: %s' % show)
+            show_name = self.config.get_output(show_name)
+            log.debug('Using config output name: %s' % show_name)
         except ShowNotInConfigException:
-            show = self.library.get_show()
+            show_name = self.library.show
             log.debug('Using the formatted show name retrieved by the library:'
-                        ' %s' % show)
+                        ' %s' % show_name)
 
         if override is not None:
-            show = override
-            log.debug('Overrode show name with: %s' % show)
+            show_name = override
+            log.debug('Overrode show name with: %s' % show_name)
 
         if the is True:
-            show = self.__move_leading_the_to_trailing_the(show)
+            show_name = self.__move_leading_the_to_trailing_the(show_name)
 
-        log.debug('Final show name: %s' % show)
+        log.debug('Final show name: %s' % show_name)
 
-        return show
+        return show_name
 
-    def build_path(self, rename_dir=None, organise=None, format=None, **kwargs):
+    def build_path(self, episode, rename_dir=None, organise=None, format=None):
         """
         Build the full destination path and filename of the renamed file Set
         the output format for the file name of a renamed show.
@@ -162,36 +177,37 @@ class TvRenamr():
         name.
         :rtype: A string.
         """
-        kwargs['show'] = self.__clean_names(kwargs['show']\
-                .replace(kwargs['show'][:1], kwargs['show'][:1].upper(), 1))
-        if len(kwargs['episode']) == 1:
-            kwargs['episode'] = '0' + kwargs['episode']
+        episode.show = self.__clean_names(episode.show)
+        episode.title = self.__clean_names(episode.title, '/',)
+        if len(episode.episode) == 1:
+            episode.episode = '0' + episode.episode
 
         if format is None:
-            format = self.config.get(kwargs['show'], 'format')
+            format = self.config.get(episode.show, 'format')
         if '%x' not in format:
             format = format + '%x'
-        format = format.replace('%n', kwargs['show'])\
-                        .replace('%s', str(int(kwargs['season'])))\
-                        .replace('%e', kwargs['episode'])\
-                        .replace('%t', self.__clean_names(kwargs['title'])) \
-                        .replace('%x', kwargs['extension'])
+        format = format.replace('%n', episode.show)\
+                        .replace('%s', str(int(episode.season)))\
+                        .replace('%e', episode.episode)\
+                        .replace('%t', self.__clean_names(episode.title))\
+                        .replace('%x', episode.extension)
 
         if rename_dir is None:
-            rename_dir = self.config.get(kwargs['show'], 'renamed')
+            rename_dir = self.config.get(episode.show, 'renamed')
         if rename_dir is False:
             rename_dir = self.working_dir
 
         if organise is None:
-            organise = self.config.get(kwargs['show'], 'organise')
+            organise = self.config.get(episode.show, 'organise')
 
         if organise is True:
             rename_dir = self.__build_organise_path(rename_dir, \
-                                    kwargs['show'], kwargs['season'])
+                                    episode.show, episode.season)
 
         log.log(22, 'Directory: %s' % rename_dir)
-        log.debug('Full path: %s' % rename_dir + format)
-        return os.path.join(rename_dir, format)
+        path = os.path.join(rename_dir, format)
+        log.debug('Full path: %s' % path)
+        return path
 
     def rename(self, current_filepath, destination_filepath):
         """
@@ -214,8 +230,7 @@ class TvRenamr():
             destination_file = os.path.split(destination_filepath)[1]
             log.log(26, 'Renamed: \"%s\"' % destination_file)
         else:
-            raise EpisodeAlreadyExistsInDirectoryException(\
-                                                        destination_filepath)
+            raise EpisodeAlreadyExistsInDirectoryException(destination_filepath)
 
     def __build_organise_path(self, start_path, show_name, season_number):
         """
@@ -253,12 +268,12 @@ class TvRenamr():
         :returns: An actual regular expression.
         :rtype: A string.
         """
-        series = '(?P<show>[\w\s.,_-]+)'
+        series = '(?P<show>[\w\s.,_-]+?)'
         season = '(?P<season>[\d]{1,2})'
         episode = '(?P<episode>[\d]{2})'
 
         if regex is None:
-            return series + '\.[Ss]?' + season + '[XxEe]?' + episode
+            return series + '\.[Ss]?' + season + '[XxEe]' + episode + '\.'
         if regex.find('%s') is -1 or regex.find('%e') is -1:
             raise IncorrectCustomRegularExpressionSyntaxException(regex)
 
@@ -297,8 +312,7 @@ class TvRenamr():
 
         return regex
 
-    def __clean_names(self, filename, character_to_replace=':', \
-                        replacement_character=','):
+    def __clean_names(self, filename, before=':', after=','):
         """
         Cleans the string passed in, making it be safe for all file systems.
         Also allows the user to specify the new characters to be used.
@@ -306,16 +320,16 @@ class TvRenamr():
         :param fn: The string to replace characters in.
         :type fn: A string.
 
-        :param character_to_replace: The character to replace.
-        :type character_to_replace: A string that defaults to a colon ':'.
+        :param original: The character to replace.
+        :type original: A string that defaults to a colon ':'.
 
-        :param replacement_character: The replacement character.
-        :type replacement_character: A string that defaults to a comma ','.
+        :param after: The replacement character.
+        :type after: A string that defaults to a comma ','.
 
         :returns: The file
         :rtype: A string.
         """
-        return filename.replace(character_to_replace, replacement_character)
+        return filename.replace(before, after)
 
     def __move_leading_the_to_trailing_the(self, show_name):
         """
