@@ -1,13 +1,12 @@
 #!/usr/bin/python
 
-__author__ = 'George Hickman'
-__version__ = '2.2.7'
-
+import logging
 import os
 import sys
-from optparse import OptionParser, SUPPRESS_HELP
 
+from __init__ import get_version
 from config import Config
+from episode import Episode
 from errors import *
 from logs import start_logging
 from main import TvRenamr
@@ -15,8 +14,7 @@ from options import OptionParser
 
 log = logging.getLogger('Core')
 
-parser = OptionParser(usage="tvr [options] <file/folder>",
-                        version="Tv Renamr %s" % __version__)
+parser = OptionParser(usage='tvr [options] <file/folder>', version='Tv Renamr %s' % get_version())
 options, args = parser.parse_args()
 
 class FrontEnd():
@@ -33,6 +31,7 @@ class FrontEnd():
             os.path.join(sys.path[0], 'config.yml'))
 
         # get the first viable config from the list of possibles
+        self.config = None
         for config in possible_config:
             if config is not None and os.path.exists(config):
                 self.config = Config(config)
@@ -111,25 +110,20 @@ class FrontEnd():
 
         try:
             tv = TvRenamr(working, self.config, options.debug, options.dry)
-            credentials = tv.extract_details_from_file(filename, \
-                                                    user_regex=options.regex)
+            episode = Episode()
+            episode.show, episode.season, episode.episode, episode.extension = tv.extract_details_from_file(filename, user_regex=options.regex)
+            if options.show:
+                episode.show = options.show
             if options.season:
-                credentials['season'] = options.season
+                episode.season = options.season
             if options.episode:
-                credentials['episode'] = options.episode
+                episode.episode = options.episode
 
-            credentials['title'] = tv.retrieve_episode_name( \
-                                                library=options.library, \
-                                                canonical=options.canonical, \
-                                                **credentials)
-            credentials['show'] = tv.format_show_name( \
-                                    show=credentials['show'], the=options.the,\
-                                    override=options.name)
-            path = tv.build_path(rename_dir=options.rename_dir, \
-                                organise=options.organise, \
-                                format=options.output_format, **credentials)
+            episode.title = tv.retrieve_episode_name(episode, library=options.library, canonical=options.canonical)
+            episode.show = tv.format_show_name(episode.show, the=options.the, override=options.show_override)
+            path = tv.build_path(episode, rename_dir=options.rename_dir, organise=options.organise, format=options.output_format)
             tv.rename(filename, path)
-        except (ConfigNotFoundException, NoNetworkConnectionException):
+        except (ConfigNotFoundException, NoNetworkConnectionException, NoMoreLibrariesException):
             if options.dry or options.debug:
                 self.__stop_dry_run()
             exit()
@@ -172,3 +166,4 @@ def run():
 
 if __name__ == "__main__":
     run()
+
